from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import CURVE_LEGEND, ON, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .LineStyle import LineStyle
from .SymbolStyle import SymbolStyle
from .XYData import XYData


@abaqus_class_doc
class XYCurve:
    """The XYCurve object is used to plot **X - Y** data and to store its display attributes.

    .. note::
        This object can be accessed by::

            import visualization
            session.charts[name].axes1[i].axisData.curves[i]
            session.charts[name].axes2[i].axisData.curves[i]
            session.charts[name].curves[name]
            session.curves[name]
            session.defaultChartOptions.defaultAxis1Options.axisData.curves[i]
            session.defaultChartOptions.defaultAxis2Options.axisData.curves[i]
            session.xyPlots[name].charts[name].axes1[i].axisData.curves[i]
            session.xyPlots[name].charts[name].axes2[i].axisData.curves[i]
            session.xyPlots[name].charts[name].curves[name]
            session.xyPlots[name].curves[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: An Int specifying the frequency of plotting the markers. Possible values are
    #: **symbolFrequency** > 0. If **symbolFrequency** = 1, then markers are plotted at every point.
    #: The default value is 1.
    symbolFrequency: int = 1

    #: A Boolean specifying whether to use the system supplied legend label. The default value
    #: is ON.
    useDefault: Boolean = ON

    #: A SymbolicConstant specifying how the system supplied, default legend label is to be
    #: generated. Possible values are CURVE_LEGEND, CURVE_NAME, and CURVE_NAME_LEGEND. The
    #: default value is CURVE_LEGEND.
    legendSource: SymbolicConstant = CURVE_LEGEND

    #: An XYData object specifying the data for the curve.
    data: Optional[XYData] = None

    #: A LineStyle object specifying the line properties used to display the curve.
    lineStyle: LineStyle = LineStyle()

    #: A SymbolStyle object specifying the symbol properties used to display the curve.
    symbolStyle: SymbolStyle = SymbolStyle()

    #: A String specifying the label to be displayed in the legend. By default, the label is
    #: system defined.
    legendLabel: str = ""

    #: A tuple of SymbolicConstants specifying that describe how curves are to be displayed.
    #: Possible values are LINE and SYMBOL. The default value is (LINE).
    displayTypes: Optional[SymbolicConstant] = None

    @abaqus_method_doc
    def Curve(self, name: str, data: XYData):
        """This method creates an XYCurve object from an XYData object.

        .. note::
            This function can be accessed by::

                session.Curve

        Parameters
        ----------
        name
            A String specifying the repository key.
        data
            An XYData object specifying the data for the curve.

        Returns
        -------
        XYCurve
            An XYCurve object.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        displayTypes: Optional[Literal[C.SYMBOL, C.LINE]] = None,
        legendLabel: str = "",
        symbolFrequency: int = 1,
        useDefault: Boolean = ON,
    ):
        """This method modifies the XYCurve object.

        Parameters
        ----------
        displayTypes
            A sequence of SymbolicConstants specifying that describe how curves are to be displayed.
            Possible values are LINE and SYMBOL. The default value is (LINE).
        legendLabel
            A String specifying the label to be displayed in the legend. By default, the label is
            system defined.
        symbolFrequency
            An Int specifying the frequency of plotting the markers. Possible values are
            **symbolFrequency** > 0. If **symbolFrequency** = 1, then markers are plotted at every point.
            The default value is 1.
        useDefault
            A Boolean specifying whether to use the system supplied legend label. The default value
            is ON.
        """
        ...



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class RegionPairs:
    """The RegionPairs object stores the domain pair definition for ContactExp and ContactStd objects. The
    RegionPairs object has no constructor or members.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].excludedPairs
            mdb.models[name].interactions[name].includedPairs

        The corresponding analysis keywords are:

        - CONTACT INCLUSIONS
        - CONTACT EXCLUSIONS
    """

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        useAllstar: Boolean = OFF,
        addPairs: Optional[Literal[C.SELF, C.GLOBAL]] = None,
        removePairs: Optional[Literal[C.SELF, C.GLOBAL]] = None,
    ):
        """This method allows addition and removal of domain pairs in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the region pair assignments are to be
            modified.
        useAllstar
            A Boolean specifying whether the contacting surface pair consists of all exterior faces
            and -- in an Abaqus/Explicit analysis -- analytical rigid surfaces, shell edges, and
            beam segments in the model.
        addPairs
            A sequence of pairs of region objects or SymbolicConstants that specify the surface
            pairs to add to the included pairs of the ContactExp or ContactStd object in the given
            step. Possible values of the SymbolicConstants are GLOBAL and SELF. When used with a
            ContactExp object, the second parameter of each pair can also be a string that
            references an Eulerian material surface.
        removePairs
            A sequence of pairs of region objects or SymbolicConstants that specify the surface
            pairs to remove from the included pairs of the ContactExp or ContactStd object in the
            given step. Possible values of the SymbolicConstants are GLOBAL and SELF. When used with
            a ContactExp object, the second parameter of each pair can also be a string that
            references an Eulerian material surface.
        """
        ...



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import (
    MINIMUM_MOVE,
    ON,
    REGULAR,
    STANDARD,
    Boolean,
)
from ...UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ConstrainedSketchOptions:
    """The ConstrainedSketchOptions object is used to store values and attributes associated with a particular
    sketch. The ConstrainedSketchOptions object has no constructor.

    .. note::
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name].sketchOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        sheetSize: Optional[float] = None,
        gridSpacing: Optional[float] = None,
        grid: Boolean = ON,
        gridFrequency: int = 1,
        dimensionTextHeight: float = 12,
        decimalPlaces: Optional[int] = None,
        constructionGeometry: Boolean = ON,
        gridSnap: Boolean = ON,
        preselection: Boolean = ON,
        sheetAuto: Boolean = ON,
        gridOrigin: tuple = (),
        gridAngle: float = 0,
        viewStyle: Literal[C.AXISYM, C.REGULAR] = REGULAR,
        addImpliedConstraints: Boolean = ON,
        maxCoplanarEntities: int = 300,
        autoConstrainAngularTolerance: float = 0,
        autoConstrainLinearTolerance: Optional[float] = None,
        autoConstrainOptions: Optional[
            Literal[C.PERPENDICULAR, C.EQUALRADIUS, C.IDENTICAL, C.TANGENT, C.PARALLEL, C.CONCENTRIC]
        ] = None,
        dragMethod: Literal[C.WEIGHTED, C.RELAXATION, C.MINIMUM_MOVE, C.STANDARD] = MINIMUM_MOVE,
        editMethod: Literal[C.WEIGHTED, C.RELAXATION, C.MINIMUM_MOVE, C.STANDARD] = STANDARD,
    ):
        """This method modifies the ConstrainedSketchOptions object.

        Parameters
        ----------
        sheetSize
            A Float specifying the sheet size. Possible values are Floats > 0. The default value is
            the **sheetSize** specified with the Sketch method.
        gridSpacing
            A Float specifying the spacing between gridlines. Possible values are Floats > 0. The
            default value is approximately 2.5% of **sheetSize**.
        grid
            A Boolean specifying whether the grid is shown. The default value is ON.
        gridFrequency
            An Int specifying how often gridlines are shown. Possible values are **gridFrequency** >
            0. The default value is 1.
        dimensionTextHeight
            A Float specifying the height of the dimension text in points. Possible values are
            Floats > 0. The default value is 12.0.
        decimalPlaces
            An Int specifying how many decimal places are shown in dimensions. Possible values are 0
            ≤ **decimalPlaces** ≤ 6. The initial value depends on the value of **sheetSize**.
        constructionGeometry
            A Boolean specifying whether construction geometry is shown. The default value is ON.
        gridSnap
            A Boolean specifying whether the cursor snaps to the grid. The default value is ON.
        preselection
            A Boolean specifying whether geometry will be preselected. The default value is ON.
        sheetAuto
            A Boolean specifying if the sheet size and the grid spacing are automatically computed.
            The default value is ON.
        gridOrigin
            A sequence of Floats specifying the **X - Y** coordinates for the origin of the grid. The
            default value is (0, 0).
        gridAngle
            A Float specifying the angle of the grid relative to the computer screen. The default
            value is 0.0.
        viewStyle
            A SymbolicConstant specifying the type of sketch displayed in the viewport. Possible
            values are REGULAR and AXISYM. The default value is REGULAR.
        addImpliedConstraints
            A Boolean specifying if implied constraints are added during sketching. The default
            value is ON.
        maxCoplanarEntities
            An Int specifying the maximum number of coplanar entities which should be automatically
            projected from the background, when a sketch based feature is created or edited. When
            this value is exceeded no entities are automatically projected and a warning issued.
            Possible values are **maxCoplanarEntities** > 0. The default value is 300.
        autoConstrainAngularTolerance
            A Float specifying the angular tolerance in degrees which is used to determine parallel
            and tangential conditions during the auto-constrain operation. For example any two lines
            which have an angle smaller than the given **autoConstrainAngularTolerance** will be
            assumed to be parallel, and a parallel constrain may be added during the auto-constrain
            operation. The default value is 0.01.
        autoConstrainLinearTolerance
            A Float specifying the linear tolerance which is used to determine when two points or
            geometries are coincident during the auto-constrain operation. The default value is
            10⁻⁶.
        autoConstrainOptions
            A sequence of SymbolicConstants specifying which type of constraints may be added by the
            auto-constraint tool. Possible values are PARALLEL, PERPENDICULAR, IDENTICAL, TANGENT,
            CONCENTRIC, and EQUALRADIUS. The default value is (PARALLEL,, PERPENDICULAR,,
            IDENTICAL,, TANGENT,, CONCENTRIC,, EQUALRADIUS).
        dragMethod
            A SymbolicConstant specifying the constraint solving mode used by the sketcher during
            drag operation. Possible values are MINIMUM_MOVE, STANDARD, WEIGHTED, and RELAXATION.
            The default value is MINIMUM_MOVE.
        editMethod
            A SymbolicConstant specifying the constraint solving mode used by the sketcher during
            regular sketch editing and adding new constraints and dimensions. Possible values are
            MINIMUM_MOVE, STANDARD, WEIGHTED, and RELAXATION. The default value is STANDARD.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc

from .Area import Area
from .Title import Title


@abaqus_class_doc
class DefaultPlot:
    """The DefaultPlot object is used to hold on default plot attributes. The DefaultPlot object attributes are
    used whenever an XYPlot object is created. A DefaultPlot object is automatically created when opening a
    session.

    .. note::
        This object can be accessed by::

            import visualization
            session.defaultPlot
    """

    #: An Area object specifying an Area used to hold on to the default display properties for
    #: the plot area.
    area: Area = Area()

    #: A Title object specifying a Title object used to hold on to the default properties of
    #: the XY-Plot title.
    title: Title = Title()



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Step.StepBase import StepBase
from ..TableCollection.ActivateElements import ActivateElements


@abaqus_class_doc
class TableCollectionStep(StepBase):
    """The Step object stores the parameters that determine the context of the step. The Step object is the
    abstract base type for other Step objects. The Step object has no explicit constructor. The methods and
    members of the Step object are common to all objects derived from the Step.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]

    .. versionadded:: 2020
        The ``TableCollectionStep`` class was added.
    """

    @abaqus_method_doc
    def ActivateElements(
        self,
        tableCollection: str,
        activation: str,
        eigenTimeConst: str = "",
        expansionTimeConstant: str = "",
    ) -> ActivateElements:
        """This method creates an ActivateElements object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ActivateElements

        Parameters
        ----------
        tableCollection
            A String specifying the name of the tableCollection object.
        activation
            A string specifying the name of progressive element activation.
        eigenTimeConst
            A Double specifying the time constant used to ramp up the eigenstrains at element
            activation.
        expansionTimeConstant
            A Double specifying the time constant used to ramp up the thermal strains at element
            activation.

        Returns
        -------
        elements: ActivateElements
            An ActivateElements object.
        """
        self.activateElements["activation"] = activateElements = ActivateElements(
            tableCollection, activation, eigenTimeConst, expansionTimeConstant
        )
        return activateElements



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import UNIFORM, UNSET, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Load import Load


@abaqus_class_doc
class Pressure(Load):
    """The Pressure object defines a pressure load. The Pressure object is derived from the Load object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the pressure is distributed spatially. Possible values
    #: are UNIFORM, USER_DEFINED, FIELD, HYDROSTATIC, STAGNATION, VISCOUS, TOTAL_FORCE, and
    #: DISCRETE_FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A String specifying the name of the AnalyticalField or DiscreteField object associated
    #: with this load. The **field** argument applies only when **distributionType** = FIELD or
    #: **distributionType** = DISCRETE_FIELD. The default value is an empty string.
    field: str = ""

    #: A Region object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float = 0.0,
        hZero: float = 0.0,
        hReference: float = 0.0,
        field: str = "",
        refPoint: str = "",
        distributionType: Literal[
            C.FIELD, C.DISCRETE_FIELD, C.STAGNATION, C.VISCOUS, C.TOTAL_FORCE, C.HYDROSTATIC, C.UNIFORM, C.USER_DEFINED
        ] = UNIFORM,
        amplitude: str = UNSET,
    ):
        """This method creates a Pressure object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Pressure

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the pressure is created.
        region
            A Region object specifying the region to which the load is applied.
        magnitude
            A Float or a Complex specifying the pressure magnitude. Note: *magnitude* is optional if
            **distributionType** = USER_DEFINED.
        hZero
            A Float specifying the height of the zero pressure level when
            **distributionType** = HYDROSTATIC.
        hReference
            A Float specifying the height of the reference pressure level when
            **distributionType** = HYDROSTATIC.
        field
            A String specifying the name of the AnalyticalField or DiscreteField object associated
            with this load. The **field** argument applies only when **distributionType** = FIELD or
            **distributionType** = DISCRETE_FIELD. The default value is an empty string.
        refPoint
            A Region specifying the reference point from which the relative velocity is determined
            when **distributionType** = STAGNATION or VISCOUS.
        distributionType
            A SymbolicConstant specifying how the pressure is distributed spatially. Possible values
            are UNIFORM, USER_DEFINED, FIELD, HYDROSTATIC, STAGNATION, VISCOUS, TOTAL_FORCE, and
            DISCRETE_FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.

        Returns
        -------
        Pressure
            A Pressure object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        field: str = "",
        refPoint: str = "",
        distributionType: Literal[
            C.FIELD, C.DISCRETE_FIELD, C.STAGNATION, C.VISCOUS, C.TOTAL_FORCE, C.HYDROSTATIC, C.UNIFORM, C.USER_DEFINED
        ] = UNIFORM,
        amplitude: str = UNSET,
    ):
        """This method modifies the data for an existing Pressure object in the step where it is created.

        Parameters
        ----------
        field
            A String specifying the name of the AnalyticalField or DiscreteField object associated
            with this load. The **field** argument applies only when **distributionType** = FIELD or
            **distributionType** = DISCRETE_FIELD. The default value is an empty string.
        refPoint
            A Region specifying the reference point from which the relative velocity is determined
            when **distributionType** = STAGNATION or VISCOUS.
        distributionType
            A SymbolicConstant specifying how the pressure is distributed spatially. Possible values
            are UNIFORM, USER_DEFINED, FIELD, HYDROSTATIC, STAGNATION, VISCOUS, TOTAL_FORCE, and
            DISCRETE_FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        magnitude: Optional[float] = None,
        hZero: Optional[float] = None,
        hReference: Optional[float] = None,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing Pressure object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        magnitude
            A Float or a Complex specifying the pressure magnitude.
        hZero
            A Float specifying the height of the zero pressure level when
            **distributionType** = HYDROSTATIC.
        hReference
            A Float specifying the height of the reference pressure level when
            **distributionType** = HYDROSTATIC.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            load has no amplitude reference. You should provide the **amplitude** argument only if it
            is valid for the specified step.
        """
        ...



from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import ON, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class SurfaceFeatureAssignment:
    """The SurfaceFeatureAssignment object stores the surface feature angle assignment definition for surfaces
    in ContactExp or ContactStd objects. The SurfaceFeatureAssignment object has no constructor or members.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].surfaceFeatureAssignments

        The corresponding analysis keywords are:

        - SURFACE PROPERTY ASSIGNMENT
    """

    @abaqus_method_doc
    def changeValuesInStep(
        self,
        stepName: str,
        index: int,
        value: Union[Literal[C.GLOBAL, C.ALL, C.CURRENT, C.ORIGINAL, C.NONE, C.PICKED, C.PERIMETER], float],
        useDynFeatEdge: Boolean = ON,
    ):
        """This method allows modification of surface feature angle assignments already defined on surfaces in a
        given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the surface feature assignments are to
            be modified.
        index
            An Int specifying the position of the surface feature angle assignment whose value is to
            be modified.
        value
            A tuple specifying the value of the surface feature assignments for the surface whose
            index is referenced. Each tuple contains two entries for Abaqus/Standard and three
            entries for Abaqus/Explicit:

            - A Float or a SymbolicConstant specifying the overriding surface primary feature angle
              value to be used for the surface. Possible values of the SymbolicConstant are PERIMETER,
              ALL, PICKED, or NONE. The ALL and PICKED values cannot be specified with the GLOBAL
              region constant and can be used only in the Abaqus/Explicit version of general contact.
            - A Float or a SymbolicConstant specifying the overriding surface secondary feature
              angle value to be used for the surface. Possible values of the SymbolicConstant are
              PERIMETER, ALL, PICKED, or NONE. The ALL and PICKED values cannot be specified with the
              GLOBAL region constant and can be used only in the Abaqus/Explicit version of general
              contact.
            - A SymbolicConstant ORIGINAL or CURRENT specifying the configuration.
        useDynFeatEdge
            A Boolean specifying whether the dynamic feature edge criterion should be activated for the contact surface
            in an Abaqus/Explicit analysis. This argument can be set to ON (default) or OFF.

            .. versionadded:: 2023

                The ``useDynFeatEdge`` argument was added.
        """
        ...

    @abaqus_method_doc
    def appendInStep(
        self,
        stepName: str,
        assignments: Union[Literal[C.GLOBAL, C.ALL, C.CURRENT, C.ORIGINAL, C.NONE, C.PICKED, C.PERIMETER], float],
        useDynFeatEdge: Boolean = ON,
    ):
        """This method allows addition of surface feature angle assignments to new surfaces in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which new surface feature angle assignments
            are to be defined.
        assignments
            A sequence of tuples specifying the surface feature angle assignments. Each tuple
            contains three entries for Abaqus/Standard and four entries for Abaqus/Explicit:

            - A region or a material object or the SymbolicConstant GLOBAL specifying the surface to
              which the feature angle is assigned.
            - A Float or a SymbolicConstant specifying the overriding surface primary feature angle
              value to be used for the surface. Possible values of the SymbolicConstant are PERIMETER,
              ALL, PICKED, or NONE. The ALL and PICKED values cannot be specified with the GLOBAL
              region constant and can be used only in the Abaqus/Explicit version of general contact.
            - A Float or a SymbolicConstant specifying the overriding surface slave feature
              angle value to be used for the surface. Possible values of the SymbolicConstant are
              PERIMETER, ALL, PICKED, or NONE. The ALL and PICKED values cannot be specified with the
              GLOBAL region constant and can be used only in the Abaqus/Explicit version of general
              contact.
            - A SymbolicConstant ORIGINAL or CURRENT specifying the configuration.
        useDynFeatEdge
            A Boolean specifying whether the dynamic feature edge criterion should be activated for the contact surface
            in an Abaqus/Explicit analysis. This argument can be set to ON (default) or OFF.

            .. versionadded:: 2023

                The ``useDynFeatEdge`` argument was added.
        """
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """The delete method allows you to delete existing surface feature angle assignments from a ContactExp
        object.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each surface feature angle assignment to
            delete.
        """
        ...



from typing import Sequence

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import OFF, Boolean
from .Imperfection import Imperfection


class FileImperfection(Imperfection):
    """The FileImperfection object defines geometric imperfection on an assembly region. The FileImperfection
    object is derived from the Imperfection object.

    .. note::
        This object can be accessed by::

            import assembly
            mdb.models[name].rootAssembly.engineeringFeatures.imperfections[name]

        The corresponding analysis keywords are:

        - IMPERFECTION

        The table data for this object are:

        - Mode Number.
        - Scaling factor for this mode.

    .. versionadded:: 2023
        The ``FileImperfection`` class was added.
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the results file from a previous analysis from which the file imperfection is applied.
    file: str

    #: An Int specifying the step number (in the analysis whose file is being used as input to this option) from
    #: which the modal or displacement data are to be read.
    step: int

    #: A sequence of sequences of Integers and Floats specifying linearSuperpositions. The items in the table data
    #: are described below.
    linearSuperpositions: Sequence[Sequence[int]]

    #: A Region object specifying the region to which the file imperfection is applied. By default, the
    #: imperfection will be applied to all nodes in the model.
    region: Region = Region()

    #: An Int specifying the increment number (in the analysis whose file is being used as input to this option)
    #: from which the displacement data are to be read. By default, the data will be read from the last increment
    #: available for the specified step.
    increment: int = -1

    #: A Boolean specifying whether the imperfection is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    def __init__(
        self,
        name: str,
        file: str,
        step: int,
        linearSuperpositions: Sequence[Sequence[int]],
        region: Region = Region(),
        increment: int = -1,
    ):
        """This method creates a FileImperfection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].rootAssembly.engineeringFeatures.FileImperfection

        Parameters
        ----------
        name
            A String specifying the repository key.
        file
            A String specifying the results file from a previous analysis from which the file imperfection is applied.
        step
            An Int specifying the step number (in the analysis whose file is being used as input to this option) from
            which the modal or displacement data are to be read.
        linearSuperpositions
            A sequence of sequences of Integers and Floats specifying linearSuperpositions. The items in the table data
            are described below.
        region
            A Region object specifying the region to which the file imperfection is applied. By default, the
            imperfection will be applied to all nodes in the model.
        increment
            An Int specifying the increment number (in the analysis whose file is being used as input to this option)
            from which the displacement data are to be read. By default, the data will be read from the last increment
            available for the specified step.

        Returns
        -------
        FileImperfection
            A FileImperfection object.
        """
        super().__init__(name)

    def setValues(
        self,
        file: str,
        step: int,
        linearSuperpositions: Sequence[Sequence[int]],
        region: Region = Region(),
        increment: int = -1,
    ):
        """This method modifies the FileImperfection object.

        Parameters
        ----------
        file
            A String specifying the results file from a previous analysis from which the file imperfection is applied.
        step
            An Int specifying the step number (in the analysis whose file is being used as input to this option) from
            which the modal or displacement data are to be read.
        linearSuperpositions
            A sequence of sequences of Integers and Floats specifying linearSuperpositions. The items in the table data
            are described below.
        region
            A Region object specifying the region to which the file imperfection is applied. By default, the
            imperfection will be applied to all nodes in the model.
        increment
            An Int specifying the increment number (in the analysis whose file is being used as input to this option)
            from which the displacement data are to be read. By default, the data will be read from the last increment
            available for the specified step.
        """
        ...



from typing import Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import OFF, Boolean
from ..ConstrainedSketchVertex.ConstrainedSketchVertex import ConstrainedSketchVertex
from .ConstrainedSketchDimension import ConstrainedSketchDimension


@abaqus_class_doc
class VerticalDimension(ConstrainedSketchDimension):
    @abaqus_method_doc
    def __init__(
        self,
        vertex1: ConstrainedSketchVertex,
        vertex2: ConstrainedSketchVertex,
        textPoint: Sequence[float],
        value: Optional[float] = None,
        reference: Boolean = OFF,
    ):
        """This method constructs a ConstrainedSketchDimension between two vertices. A vertical dimension
        controls the vertical distance along the **Y** axis between two vertices.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].VerticalDimension

        Parameters
        ----------
        vertex1
            A ConstrainedSketchVertex object specifying the first endpoint.
        vertex2
            A ConstrainedSketchVertex object specifying the second endpoint.
        textPoint
            A pair of Floats specifying the location of the dimension text.
        value
            A Float specifying the angle between the two lines.
        reference
            A Boolean specifying whether the created dimension enforces the above value or if it
            simply measures the angle between two lines.

        Returns
        -------
        ConstrainedSketchDimension
            A ConstrainedSketchDimension object (None if the dimension cannot be created).
        """
        ...



"""
Compression Model
=================

A simple example to establish a compression model.

.. figure:: /images/model.png
    :width: 50%
    :align: center

    A simple compression model.
"""

from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import *

executeOnCaeStartup()

# Model
model = mdb.models["Model-1"]

# Part
sketch = model.ConstrainedSketch(name="sketch", sheetSize=1.0)
sketch.rectangle((0, 0), (1, 1))
part = model.Part(name="part", dimensionality=THREE_D, type=DEFORMABLE_BODY)
part.BaseSolidExtrude(sketch=sketch, depth=1)

# Create sets
part.Set(name="set-all", cells=part.cells.findAt(coordinates=((0.5, 0.5, 0.5),)))
part.Set(name="set-bottom", faces=part.faces.findAt(coordinates=((0.5, 0.5, 0.0),)))
part.Set(name="set-top", faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))
part.Surface(name="surface-top", side1Faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))

# Assembly
model.rootAssembly.DatumCsysByDefault(CARTESIAN)
model.rootAssembly.Instance(name="instance", part=part, dependent=ON)

# Material
material = model.Material(name="material")
material.Elastic(table=((1000, 0.2),))
material.Density(table=((2500,),))

# Section
model.HomogeneousSolidSection(name="section", material="material", thickness=None)
part.SectionAssignment(region=part.sets["set-all"], sectionName="section")

# Step
step = model.StaticStep(
    name="Step-1",
    previous="Initial",
    description="",
    timePeriod=1.0,
    timeIncrementationMethod=AUTOMATIC,
    maxNumInc=100,
    initialInc=0.01,
    minInc=0.001,
    maxInc=0.1,
)

# Output request
field = model.FieldOutputRequest("F-Output-1", createStepName="Step-1", variables=("S", "E", "U"))

# Boundary condition
bottom_instance = model.rootAssembly.instances["instance"].sets["set-bottom"]
bc = model.DisplacementBC(
    name="BC-1", createStepName="Initial", region=bottom_instance, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET
)

# Load
top_instance = model.rootAssembly.instances["instance"].surfaces["surface-top"]
pressure = model.Pressure("pressure", createStepName="Step-1", region=top_instance, magnitude=100)

# Mesh
elem1 = mesh.ElemType(elemCode=C3D8R)
elem2 = mesh.ElemType(elemCode=C3D6)
elem3 = mesh.ElemType(elemCode=C3D4)
part.setElementType(regions=(part.cells,), elemTypes=(elem1, elem2, elem3))
part.seedPart(size=0.1)
part.generateMesh()

# Job
job = mdb.Job(name="Job-1", model="Model-1")
job.writeInput()

# Submit the job
job.submit()
job.waitForCompletion()

# Save abaqus model
mdb.saveAs("compression.cae")



import ast
import os
import sys

from .cli import abaqus

ABAQUS_COMMAND_OPTIONS = {"noGUI": True}


def run(cae: bool = True) -> None:
    """Runs Abaqus command in system's CLI.

    This function uses the top level script file to run the Abaqus
    command, and the arguments passed to it

    Parameters
    ----------
    cae : bool, optional
        Whether to use `abaqus cae` command, True for `abaqus cae`, False for `abaqus python`, by default True
    """
    # check if in debug mode and run
    debug = os.environ.get("ABQPY_DEBUG", "false").lower() == "true"
    gettrace = getattr(sys, "gettrace", None)
    debug = debug or (gettrace is not None and gettrace())

    # Check if it is imported by sphinx to generate docs
    make_docs = os.environ.get("ABQPY_MAKE_DOCS", "false").lower() == "true"

    # If in debug mode or making docs, return
    if debug or make_docs:
        return

    # If it is a jupyter notebook, convert it to python script
    try:  # If it is a jupyter notebook
        import ipynbname

        filePath = ipynbname.path()
        os.system(f"jupyter nbconvert --to python {filePath}")
        filePath = str(filePath).replace(".ipynb", ".py")
    except (FileNotFoundError, ImportError, Exception):
        # Get the main script file
        main = sys.modules["__main__"]
        if not hasattr(main, "__file__") or main.__file__ is None:
            return
        filePath = os.path.relpath(main.__file__)

    # Alternative to use abaqus command line options at run time
    options: dict = ast.literal_eval(os.environ.get("ABAQUS_COMMAND_OPTIONS", str(ABAQUS_COMMAND_OPTIONS)))
    noGUI = options.pop("noGUI", True)
    if cae:
        options["gui"] = not noGUI
        abaqus.cae(filePath, *sys.argv[1:], **options)
    else:
        abaqus.python(filePath, *sys.argv[1:], **options)
    sys.exit(0)



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import UNIFORM, UNSET, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Load import Load


@abaqus_class_doc
class SurfaceConcentrationFlux(Load):
    """The SurfaceConcentrationFlux object defines surface concentration flux from a region or into a region.
    The SurfaceConcentrationFlux object is derived from the Load object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the surface concentration flux is distributed
    #: spatially. Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is
    #: UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A String specifying the name of the AnalyticalField object associated with this load.
    #: The **field** argument applies only when **distributionType** = FIELD. The default value is an
    #: empty string.
    field: str = ""

    #: A Region object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float,
        field: str = "",
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
    ):
        """This method creates a SurfaceConcentrationFlux object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SurfaceConcentrationFlux

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the load is created.
        region
            A Region object specifying the region to which the load is applied.
        magnitude
            A Float specifying the surface concentration flux magnitude. **magnitude** is optional if
            **distributionType** = USER_DEFINED.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        distributionType
            A SymbolicConstant specifying how the surface concentration flux is distributed
            spatially. Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is
            UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.

        Returns
        -------
        SurfaceConcentrationFlux
            A SurfaceConcentrationFlux object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        field: str = "",
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
    ):
        """This method modifies the data for an existing SurfaceConcentrationFlux object in the step where it is
        created.

        Parameters
        ----------
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        distributionType
            A SymbolicConstant specifying how the surface concentration flux is distributed
            spatially. Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is
            UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, magnitude: Optional[float] = None, amplitude: str = ""):
        """This method modifies the propagating data for an existing SurfaceConcentrationFlux object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the surface concentration flux is
            modified.
        magnitude
            A Float specifying the surface concentration flux magnitude.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            load has no amplitude reference. You should provide the **amplitude** argument only if it
            is valid for the specified step.
        """
        ...



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .._OptionsBase import _OptionsBase
from ..UtilityAndView.abaqusConstants import RLE24, SIZE_ON_SCREEN, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class QuickTimeOptions(_OptionsBase):
    """The QuickTimeOptions object is used to store values and attributes to be used in generating QuickTime
    animation. The QuickTimeOptions object has no constructor. Abaqus creates the **quickTimeOptions** member
    when the animation module is imported.

    .. note::
        This object can be accessed by::

            import animation
            session.quickTimeOptions
    """

    #: A SymbolicConstant specifying the compression method for the QuickTime format. Possible
    #: values are:
    #:
    #: - RAW24, specifying an uncompressed format of 24 bits per pixel.
    #: - RLE24, specifying a run length encoded format of 24 bits per pixel.
    #:
    #: The default value is RLE24.
    compressionMethod: SymbolicConstant = RLE24

    #: A SymbolicConstant specifying how the width and height of the image are specified.
    #: Possible values are SIZE_ON_SCREEN and USER_DEFINED. The default value is
    #: SIZE_ON_SCREEN.
    sizeDefinition: SymbolicConstant = SIZE_ON_SCREEN

    #: A pair of Ints specifying the width and height of the image in pixels when
    #: **sizeDefinition** = USER_DEFINED. Possible values are Ints in the range (*minWidth*,
    #: **minHeight**) ≤ (width, height) ≤ (*maxWidth*, **maxHeight**). The default value is the
    #: screen size.
    #: Note: The values of the minimum width (*minWidth*) and height (*minHeight*) depend on the
    #: following:viewport font size,whether decorations are printed,decoration size, and screen
    #: resolution.The minimum width and height are normally < 50 pixels. The values of the maximum
    #: width (*maxWidth*) and height (*maxHeight*) depend on the graphics capabilities of the system.
    #: The maximum width and height will be at least as large as the screen dimensions.
    imageSize: Optional[int] = None

    @abaqus_method_doc
    def setValues(
        self,
        compressionMethod: Literal[C.RLE24, C.RAW24] = RLE24,
        sizeDefinition: Literal[C.USER_DEFINED, C.SIZE_ON_SCREEN] = SIZE_ON_SCREEN,
        imageSize: Optional[int] = None,
    ):
        """This method modifies the QuickTimeOptions object.

        Parameters
        ----------
        compressionMethod
            A SymbolicConstant specifying the compression method for the QuickTime format. Possible
            values are:

            - RAW24, specifying an uncompressed format of 24 bits per pixel.
            - RLE24, specifying a run length encoded format of 24 bits per pixel.

            The default value is RLE24.
        sizeDefinition
            A SymbolicConstant specifying how the width and height of the image are specified.
            Possible values are SIZE_ON_SCREEN and USER_DEFINED. The default value is
            SIZE_ON_SCREEN.
        imageSize
            A pair of Ints specifying the width and height of the image in pixels when
            **sizeDefinition** = USER_DEFINED. Possible values are Ints in the range (*minWidth*,
            **minHeight**) ≤ (width, height) ≤ (*maxWidth*, **maxHeight**). The default value is the
            screen size.
            Note: The values of the minimum width (*minWidth*) and height (*minHeight*) depend on the
            following:viewport font size,whether decorations are printed,decoration size, and screen
            resolution.The minimum width and height are normally < 50 pixels. The values of the
            maximum width (*maxWidth*) and height (*maxHeight*) depend on the graphics capabilities
            of the system. The maximum width and height will be at least as large as the screen
            dimensions.
        """
        super().setValues(
            compressionMethod=compressionMethod,
            sizeDefinition=sizeDefinition,
            imageSize=imageSize,
        )



from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import ABSOLUTE_VALUE, FILTER_REGION
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class BeadFilter(GeometricRestriction):
    """The BeadFilter object defines a growth geometric restriction. The BeadFilter object is derived from the
    GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]

    .. versionadded:: 2023

        The ``BeadFilter`` class was added.
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A Region object specifying the region to which the geometric restriction is applied.
    region: Region

    #: A Float specifying the filter radius. The default value is double the average edge length of the model.
    radius: float = None

    #: The SymbolicConstant defines whether the filter radius is in absolute or relative units. For an absolute
    #: radius, the value is ABSOLUTE_VALUE. For a relative radius, the value is RELATIVE. The default value is
    #: ABSOLUTE_VALUE.
    filterRadiusBy: Literal[C.ABSOLUTE_VALUE, C.RELATIVE] = ABSOLUTE_VALUE

    #: The SymbolicConstant FILTER_REGION or a Region object specifying the filter check region. If the value is
    #: FILTER_REGION, the value of the region is used as both the filter region and the filter check region.
    #: The default value is FILTER_REGION.
    filterCheckRegion: Union[Literal[C.FILTER_REGION], Region] = FILTER_REGION

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        radius: float = None,
        filterRadiusBy: Literal[C.ABSOLUTE_VALUE, C.RELATIVE] = ABSOLUTE_VALUE,
        filterCheckRegion: Union[Literal[C.FILTER_REGION], Region] = FILTER_REGION,
    ):
        """This method creates a BeadFilter object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].BeadFilter

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        region
            A Region object specifying the region to which the geometric restriction is applied.
        radius
            A Float specifying the filter radius. The default value is double the average edge length of the model.
        filterRadiusBy
            The SymbolicConstant defines whether the filter radius is in absolute or relative units. For an absolute
            radius, the value is ABSOLUTE_VALUE. For a relative radius, the value is RELATIVE. The default value is
            ABSOLUTE_VALUE.
        filterCheckRegion
            The SymbolicConstant FILTER_REGION or a Region object specifying the filter check region. If the value is
            FILTER_REGION, the value of the region is used as both the filter region and the filter check region.
            The default value is FILTER_REGION.

        Returns
        -------
        BeadFilter
            A BeadFilter object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        region: Region,
        radius: float = None,
        filterRadiusBy: Literal[C.ABSOLUTE_VALUE, C.RELATIVE] = ABSOLUTE_VALUE,
        filterCheckRegion: Union[Literal[C.FILTER_REGION], Region] = FILTER_REGION,
    ):
        """This method modifies the BeadFilter object.

        Parameters
        ----------
        region
            A Region object specifying the region to which the geometric restriction is applied.
        radius
            A Float specifying the filter radius. The default value is double the average edge length of the model.
        filterRadiusBy
            The SymbolicConstant defines whether the filter radius is in absolute or relative units. For an absolute
            radius, the value is ABSOLUTE_VALUE. For a relative radius, the value is RELATIVE. The default value is
            ABSOLUTE_VALUE.
        filterCheckRegion
            The SymbolicConstant FILTER_REGION or a Region object specifying the filter check region. If the value is
            FILTER_REGION, the value of the region is used as both the filter region and the filter check region.
            The default value is FILTER_REGION.
        """
        ...



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    ON,
    SELECTIVE,
    SURFACE_TO_SURFACE,
    TWO_CONFIG,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Interaction import Interaction


@abaqus_class_doc
class SelfContactStd(Interaction):
    """The SelfContactStd object defines self-contact during an Abaqus/Standard analysis. The SelfContactStd
    object is derived from the Interaction object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name]
    """

    #: A SymbolicConstant specifying the choice of contact tracking algorithm. The STATE
    #: tracking algorithm uses only normal projections and is specified by using ONE_CONFIG.
    #: The PATH tracking algorithm uses crossing and normal projections and is specified by
    #: using TWO_CONFIG. Possible values are ONE_CONFIG and TWO_CONFIG. The default value is
    #: TWO_CONFIG.This argument is valid only when **enforcement** = SURFACE_TO_SURFACE.
    contactTracking: SymbolicConstant = TWO_CONFIG

    #: A SymbolicConstant specifying the manner in which mid-face constraints are employed.
    #: Possible values are SELECTIVE, NEVER, and ALWAYS. The default value is SELECTIVE.This
    #: argument is valid only when **enforcement** = NODE_TO_SURFACE.
    supplementaryContact: SymbolicConstant = SELECTIVE

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the SelfContactStd object is created.
    createStepName: str

    #: A Region object specifying the surface where self-contact is defined.
    surface: Region

    #: A String specifying the name of the ContactProperty object associated with this
    #: interaction.
    interactionProperty: str

    #: A SymbolicConstant specifying the discretization method. Possible values are
    #: NODE_TO_SURFACE and SURFACE_TO_SURFACE. The default value is SURFACE_TO_SURFACE.
    enforcement: SymbolicConstant = SURFACE_TO_SURFACE

    #: A Boolean specifying whether shell/membrane element thickness is considered. The default
    #: value is ON.This argument in valid only when **enforcement** = SURFACE_TO_SURFACE.
    thickness: Boolean = ON

    #: A Float specifying the degree of smoothing used for deformable or rigid main surfaces
    #: involved when **enforcement** = NODE_TO_SURFACE. The value given must lie between 0.0 and
    #: 0.5. The default value is 0.2.
    #:
    #: .. versionchanged:: 2022
    #:     Rigid master surfaces was changed to rigid main surfaces.
    smooth: float = 0

    #: A String specifying the name of the ContactControl object associated with this
    #: interaction. An empty string indicates that the default contact controls will be used.
    #: The default value is an empty string.
    contactControls: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        surface: Region,
        interactionProperty: str,
        enforcement: Literal[C.NODE_TO_SURFACE, C.SURFACE_TO_SURFACE] = SURFACE_TO_SURFACE,
        thickness: Boolean = ON,
        smooth: float = 0,
        contactControls: str = "",
    ):
        """This method creates a SelfContactStd object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SelfContactStd

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the SelfContactStd object is created.
        surface
            A Region object specifying the surface where self-contact is defined.
        interactionProperty
            A String specifying the name of the ContactProperty object associated with this
            interaction.
        enforcement
            A SymbolicConstant specifying the discretization method. Possible values are
            NODE_TO_SURFACE and SURFACE_TO_SURFACE. The default value is SURFACE_TO_SURFACE.
        thickness
            A Boolean specifying whether shell/membrane element thickness is considered. The default
            value is ON.This argument in valid only when **enforcement** = SURFACE_TO_SURFACE.
        smooth
            A Float specifying the degree of smoothing used for deformable or rigid main surfaces
            involved when **enforcement** = NODE_TO_SURFACE. The value given must lie between 0.0 and
            0.5. The default value is 0.2.

            .. versionchanged:: 2022
                Rigid master surfaces was changed to rigid main surfaces.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.

        Returns
        -------
        SelfContactStd
            A SelfContactStd object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        enforcement: Literal[C.NODE_TO_SURFACE, C.SURFACE_TO_SURFACE] = SURFACE_TO_SURFACE,
        thickness: Boolean = ON,
        smooth: float = 0,
        contactControls: str = "",
    ):
        """This method modifies the data for an existing SelfContactStd object in the step where it is created.

        Parameters
        ----------
        enforcement
            A SymbolicConstant specifying the discretization method. Possible values are
            NODE_TO_SURFACE and SURFACE_TO_SURFACE. The default value is SURFACE_TO_SURFACE.
        thickness
            A Boolean specifying whether shell/membrane element thickness is considered. The default
            value is ON.This argument in valid only when **enforcement** = SURFACE_TO_SURFACE.
        smooth
            A Float specifying the degree of smoothing used for deformable or rigid main surfaces
            involved when **enforcement** = NODE_TO_SURFACE. The value given must lie between 0.0 and
            0.5. The default value is 0.2.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, interactionProperty: str = "", contactControls: str = ""):
        """This method modifies the propagating data of an existing SelfContactStd object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the interaction is modified.
        interactionProperty
            A String specifying the name of the ContactProperty object associated with this
            interaction.
        contactControls
            A String specifying the name of the ContactControl object associated with this
            interaction. An empty string indicates that the default contact controls will be used.
            The default value is an empty string.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class LoadState:
    """The LoadState object is the abstract base type for other LoadState objects. The LoadState object has no
    explicit constructor or methods. The members of the LoadState object are common to all objects derived from
    LoadState.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]
    """

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from typing import Optional, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    OFF,
    UNCHANGED,
    UNSET,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .BoundaryCondition import BoundaryCondition


@abaqus_class_doc
class FluidCavityPressureBC(BoundaryCondition):
    """The FluidCavityPressureBC object stores the data for a fluid cavity pressure boundary condition. The
    FluidCavityPressureBC object is derived from the BoundaryCondition object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A String specifying the name of a Fluid Cavity Interaction.
    fluidCavity: str = ""

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: Optional[SymbolicConstant] = None

    #: A Region object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        fluidCavity: str,
        magnitude: float = 0,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ):
        """This method creates a FluidCavityPressureBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FluidCavityPressureBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        fluidCavity
            A String specifying the name of a Fluid Cavity Interaction.
        magnitude
            A Float specifying the fluid cavity pressure magnitude. The default value is 0.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        FluidCavityPressureBC
            A FluidCavityPressureBC object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, magnitude: float = 0, amplitude: str = UNSET, fixed: Boolean = OFF):
        """This method modifies the data for an existing FluidCavityPressureBC object in the step where it is
        created.

        Parameters
        ----------
        magnitude
            A Float specifying the fluid cavity pressure magnitude. The default value is 0.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        magnitude: Union[Literal[C.FREED], float] = UNCHANGED,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing FluidCavityPressureBC object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        magnitude
            A Float or the SymbolicConstant FREED specifying the fluid cavity pressure magnitude.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            boundary condition is changed to have no amplitude reference. You should provide the
            **amplitude** argument only if it is valid for the specified step.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class MpcConstraint:
    """The MpcConstraint object.

    .. note::
        This object can be accessed by::

            import visualization
            session.odbData[name].multiPointConstraints[i]
    """

    #: A String specifying the multipoint constraint name. This attribute is read-only.
    name: str = ""

    @abaqus_method_doc
    def constraintData(self):
        """This method returns constraint data if any are associated with the object.

        Returns
        -------
        Sequence[float]
            A tuple containing coordinates of the nodes pertaining to the constraint.
        """
        # TODO: Implement this method
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .LoadState import LoadState


@abaqus_class_doc
class ConnectorForceState(LoadState):
    """The ConnectorForceState object stores the propagating data for a connector force in a step. One instance
    of this object is created internally by the ConnectorForce object for each step. The instance is also
    deleted internally by the ConnectorForce object. The ConnectorForceState object has no constructor or
    methods. The ConnectorForceState object is derived from the LoadState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - CONNECTOR LOAD
    """

    #: A Float or a Complex specifying the connector force component in the connector's local
    #: 1-direction.
    f1: Optional[float] = None

    #: A Float or a Complex specifying the connector force component in the connector's local
    #: 2-direction.
    f2: Optional[float] = None

    #: A Float or a Complex specifying the connector force component in the connector's local
    #: 3-direction.
    f3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the connector force component in
    #: the connector's local 1-direction. Possible values are UNSET, SET, UNCHANGED, and
    #: MODIFIED.
    f1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the connector force component in
    #: the connector's local 2-direction. Possible values are UNSET, SET, UNCHANGED, and
    #: MODIFIED.
    f2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the connector force component in
    #: the connector's local 3-direction. Possible values are UNSET, SET, UNCHANGED, and
    #: MODIFIED.
    f3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from math import *  # noqa # pylint: disable=unused-import

from abqpy import run

run(cae=False)

from abaqus.Odb.OdbCommands import *  # noqa # pylint: disable=unused-import
from abaqus.UtilityAndView.BackwardCompatibility import BackwardCompatibility
from abaqusConstants import *  # noqa # pylint: disable=unused-import

backwardCompatibility = BackwardCompatibility()



from regionToolset import Region  # noqa # pylint: disable=unused-import
from sketch import Sketch  # noqa # pylint: disable=unused-import
from sketch import SketchTransform  # noqa # pylint: disable=unused-import
from xyPlot import AreaStyle  # noqa # pylint: disable=unused-import
from xyPlot import LineStyle  # noqa # pylint: disable=unused-import
from xyPlot import QuantityType  # noqa # pylint: disable=unused-import
from xyPlot import SymbolStyle  # noqa # pylint: disable=unused-import
from xyPlot import TextStyle  # noqa # pylint: disable=unused-import
from xyPlot import XYData  # noqa # pylint: disable=unused-import
from xyPlot import XYDataFromFile  # noqa # pylint: disable=unused-import
from xyPlot import XYDataFromFreeBody  # noqa # pylint: disable=unused-import
from xyPlot import XYDataFromHistory  # noqa # pylint: disable=unused-import
from xyPlot import XYDataFromPath  # noqa # pylint: disable=unused-import
from xyPlot import XYDataFromShellThickness  # noqa # pylint: disable=unused-import
from xyPlot import xyDataListFromField  # noqa # pylint: disable=unused-import

# inspected from Abaqus cli



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .BoundaryConditionState import BoundaryConditionState


@abaqus_class_doc
class DisplacementBCState(BoundaryConditionState):
    """The DisplacementBCState object stores the propagating data for a displacement/rotation boundary condition
    in a step. One instance of this object is created internally by the DisplacementBC object for each step. The
    instance is also deleted internally by the DisplacementBC object. The DisplacementBCState object has no
    constructor or methods. The DisplacementBCState object is derived from the BoundaryConditionState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BOUNDARY
    """

    #: A Float or a Complex specifying the displacement component in the 1-direction.
    u1: Optional[float] = None

    #: A Float or a Complex specifying the displacement component in the 2-direction.
    u2: Optional[float] = None

    #: A Float or a Complex specifying the displacement component in the 3-direction.
    u3: Optional[float] = None

    #: A Float or a Complex specifying the rotational displacement component about the
    #: 1-direction.
    ur1: Optional[float] = None

    #: A Float or a Complex specifying the rotational displacement component about the
    #: 2-direction.
    ur2: Optional[float] = None

    #: A Float or a Complex specifying the rotational displacement component about the
    #: 3-direction.
    ur3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the displacement component in the
    #: 1-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    u1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the displacement component in the
    #: 2-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    u2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the displacement component in the
    #: 3-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    u3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational displacement
    #: component about the 1-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and
    #: MODIFIED.
    ur1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational displacement
    #: component about the 2-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and
    #: MODIFIED.
    ur2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational displacement
    #: component about the 3-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and
    #: MODIFIED.
    ur3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..Region.Set import Set
from ..Section.CompositeShellSection import CompositeShellSection
from ..Section.GeometryShellSection import GeometryShellSection
from ..Section.HomogeneousShellSection import HomogeneousShellSection
from ..Section.SectionLayerArray import SectionLayerArray
from ..UtilityAndView.abaqusConstants import (
    AXIS_1,
    DEFAULT,
    GLOBAL,
    GRADIENT,
    NO_IDEALIZATION,
    OFF,
    ROTATION_NONE,
    SHELL,
    SIMPSON,
    UNIFORM,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .CompositePly import CompositePly
from .CompositePlyArray import CompositePlyArray
from .MaterialOrientation import MaterialOrientation


@abaqus_class_doc
class CompositeLayup:
    """The CompositeLayup object is used to specify a composite layup on a part.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].compositeLayups[i]

        The corresponding analysis keywords are:

        - SHELL SECTION
        - SHELL GENERAL SECTION
        - SOLID SECTION
    """

    #: A GeometryShellSection object.
    section: GeometryShellSection = GeometryShellSection()

    #: A MaterialOrientation object.
    orientation: MaterialOrientation = MaterialOrientation(Set(""))

    #: A CompositePlyArray object specifying the plies that make up this composite layup.
    plies: CompositePlyArray = []

    #: A String specifying the repository key.
    name: str

    #: A String specifying a description of the composite layup.
    description: str = ""

    #: A SymbolicConstant specifying the method used to define the shell offset. If
    #: **offsetType** = OFFSET_FIELD the **offsetField** argument is required. This member is valid
    #: only if **elementType** = SHELL. Possible values are SINGLE_VALUE, MIDDLE_SURFACE,
    #: TOP_SURFACE, BOTTOM_SURFACE, OFFSET_FIELD, and GLOBAL. The default value is GLOBAL.
    offsetType: SymbolicConstant = GLOBAL

    #: A String specifying The name of the field specifying the offset. This member is valid
    #: only if **elementType** = SHELL. The default value is an empty string.
    offsetField: str = ""

    #: A Float specifying The offset of the shell section. This member is valid only if
    #: **elementType** = SHELL. The default value is 0.0.
    offsetValues: float = 0

    #: A SymbolicConstant specifying the type of element in the composite layup. Possible
    #: values are SHELL, CONTINUUM_SHELL, and SOLID. The default value is SHELL.
    elementType: SymbolicConstant = SHELL

    #: A Boolean specifying whether or not the layup should be made symmetric by the analysis.
    #: The default value is OFF.
    symmetric: Boolean = OFF

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        description: str = "",
        offsetType: Literal[
            C.TOP_SURFACE, C.MIDDLE_SURFACE, C.BOTTOM_SURFACE, C.GLOBAL, C.SINGLE_VALUE, C.OFFSET_FIELD, C.SHELL
        ] = GLOBAL,
        offsetField: str = "",
        offsetValues: float = 0,
        elementType: Literal[C.SOLID, C.SHELL, C.CONTINUUM_SHELL] = SHELL,
        symmetric: Boolean = OFF,
    ):
        """This method creates a CompositeLayup object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].CompositeLayup

        Parameters
        ----------
        name
            A String specifying the repository key.
        description
            A String specifying a description of the composite layup.
        offsetType
            A SymbolicConstant specifying the method used to define the shell offset. If
            **offsetType** = OFFSET_FIELD the **offsetField** argument is required. This member is valid
            only if **elementType** = SHELL. Possible values are SINGLE_VALUE, MIDDLE_SURFACE,
            TOP_SURFACE, BOTTOM_SURFACE, OFFSET_FIELD, and GLOBAL. The default value is GLOBAL.
        offsetField
            A String specifying The name of the field specifying the offset. This member is valid
            only if **elementType** = SHELL. The default value is an empty string.
        offsetValues
            A Float specifying The offset of the shell section. This member is valid only if
            **elementType** = SHELL. The default value is 0.0.
        elementType
            A SymbolicConstant specifying the type of element in the composite layup. Possible
            values are SHELL, CONTINUUM_SHELL, and SOLID. The default value is SHELL.
        symmetric
            A Boolean specifying whether or not the layup should be made symmetric by the analysis.
            The default value is OFF.

        Returns
        -------
        CompositeLayup
            A CompositeLayup object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses a composite layup."""
        ...

    @abaqus_method_doc
    def resume(self):
        """This method resumes a composite layup that was previously suppressed."""
        ...

    @abaqus_method_doc
    def deletePlies(self):
        """This method deletes all of the plies from a composite layup."""
        ...

    @abaqus_method_doc
    def setValues(
        self,
        description: str = "",
        offsetType: Literal[
            C.TOP_SURFACE, C.MIDDLE_SURFACE, C.BOTTOM_SURFACE, C.GLOBAL, C.SINGLE_VALUE, C.OFFSET_FIELD, C.SHELL
        ] = GLOBAL,
        offsetField: str = "",
        offsetValues: float = 0,
        elementType: Literal[C.SOLID, C.SHELL, C.CONTINUUM_SHELL] = SHELL,
        symmetric: Boolean = OFF,
    ):
        """This method modifies the CompositeLayup object.

        Parameters
        ----------
        description
            A String specifying a description of the composite layup.
        offsetType
            A SymbolicConstant specifying the method used to define the shell offset. If
            **offsetType** = OFFSET_FIELD the **offsetField** argument is required. This member is valid
            only if **elementType** = SHELL. Possible values are SINGLE_VALUE, MIDDLE_SURFACE,
            TOP_SURFACE, BOTTOM_SURFACE, OFFSET_FIELD, and GLOBAL. The default value is GLOBAL.
        offsetField
            A String specifying The name of the field specifying the offset. This member is valid
            only if **elementType** = SHELL. The default value is an empty string.
        offsetValues
            A Float specifying The offset of the shell section. This member is valid only if
            **elementType** = SHELL. The default value is 0.0.
        elementType
            A SymbolicConstant specifying the type of element in the composite layup. Possible
            values are SHELL, CONTINUUM_SHELL, and SOLID. The default value is SHELL.
        symmetric
            A Boolean specifying whether or not the layup should be made symmetric by the analysis.
            The default value is OFF.
        """
        ...

    @abaqus_method_doc
    def CompositePly(
        self,
        thickness: float,
        region: Region,
        material: str,
        plyName: str,
        orientationType: Literal[C.ANGLE_45, C.ANGLE_0, C.CSYS, C.ANGLE_NEG45, C.SPECIFY_ORIENT, C.ANGLE_90],
        thicknessType: Literal[C.FIELD_THICKNESS, C.ANALYTICAL_FIELD_THICKNESS, C.SPECIFY_THICKNESS],
        orientationValue: float = 0,
        thicknessField: str = "",
        numIntPts: int = 3,
        axis: Literal[C.AXIS_1, C.AXIS_3, C.AXIS_2] = AXIS_1,
        angle: float = 0,
        additionalRotationType: Literal[C.ROTATION_NONE, C.ANGLE_0, C.ROTATION_FIELD, C.ROTATION_ANGLE] = ROTATION_NONE,
        orientation: Optional[SymbolicConstant] = None,
        additionalRotationField: str = "",
    ) -> CompositePly:
        """This method creates a CompositePly object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].CompositeLayup

        Parameters
        ----------
        thickness
            A Float specifying the thickness of the section layer.
        region
            A Region object specifying the region to which the composite ply applies.
        material
            A String specifying the name of the material for the ply.
        plyName
            A String specifying the ply identifier for this section layer. The default value is an
            empty string.
        orientationType
            A SymbolicConstant specifying the method used to define the relative orientation. If
            **orientationType** = SPECIFY_ORIENT the **orientationValue** argument is required. If
            **orientationType** = CSYS the **orientation** argument is required. Possible values are CSYS,
            SPECIFY_ORIENT, ANGLE_0, ANGLE_45, ANGLE_90, and ANGLE_NEG45. The default value is
            ANGLE_0.
        thicknessType
            A SymbolicConstant specifying the method used to define the thickness. If
            **thicknessType** = SPECIFY_THICKNESS, the **thickness** argument is required. Possible values
            are SPECIFY_THICKNESS, FIELD_THICKNESS, and ANALYTICAL_FIELD_THICKNESS. The default
            value is SPECIFY_THICKNESS.
        orientationValue
            A Float specifying the relative orientation of the section layer. The default value is
            0.0.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements and composite ply. The **thicknessField**
            argument applies when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD
            for shell elements and **thicknessType** = FIELD_THICKNESS or
            **thicknessType** = ANALYTICAL_FIELD_THICKNESS for composite ply. The default value is an
            empty string.
        numIntPts
            An Int specifying the number of integration points to be used through the section layer.
            This argument is valid only if **preIntegrate** = OFF. The default value is 3.
        axis
            A SymbolicConstant specifying the axis of a cylindrical or spherical datum coordinate
            system about which an additional rotation is applied. For shells this axis is also the
            shell normal. The **axis** argument applies only if a valid reference is provided for the
            **orientation**. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
            AXIS_1.
        angle
            A Float specifying the angle of the additional rotation. The **angle** argument applies
            only if a valid reference is provided for the **orientation**. The default value is 0.0.
        additionalRotationType
            A SymbolicConstant specifying the method used to describe the additional rotation when a
            valid orientation is specified. Use **orientationType** = ANGLE_0 and
            **additionalRotationType** = ROTATION_FIELD to specify a discrete field of rotations for
            this CompositePly. Possible values are ROTATION_NONE, ROTATION_ANGLE, and
            ROTATION_FIELD. The default value is ROTATION_NONE.
        orientation
            The SymbolicConstant None or a DatumCsys object specifying a coordinate system reference
            for the relative orientation of this layer. The default value is None.
        additionalRotationField
            A String specifying the name of the field specifying the additional rotation. The
            default value is an empty string.

        Returns
        -------
        CompositePly
            A CompositePly object.

        Raises
        ------
        AbaqusException
        """
        compositePly = CompositePly(
            thickness,
            region,
            material,
            plyName,
            orientationType,
            thicknessType,
            orientationValue,
            thicknessField,
            numIntPts,
            axis,
            angle,
            additionalRotationType,
            orientation,
            additionalRotationField,
        )
        self.plies.append(compositePly)
        return compositePly

    @abaqus_method_doc
    def CompositeShellSection(
        self,
        name: str,
        layup: SectionLayerArray,
        symmetric: Boolean = OFF,
        thicknessType: Literal[
            C.DISCRETE_FIELD, C.NODAL_ANALYTICAL_FIELD, C.ANALYTICAL_FIELD, C.UNIFORM, C.NODAL_DISCRETE_FIELD
        ] = UNIFORM,
        preIntegrate: Boolean = OFF,
        poissonDefinition: Literal[C.DEFAULT] = DEFAULT,
        poisson: float = 0,
        integrationRule: Literal[C.GAUSS, C.SIMPSON] = SIMPSON,
        temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT,
        idealization: Literal[C.SMEAR_ALL_LAYERS, C.BENDING, C.MEMBRANE, C.NO_IDEALIZATION] = NO_IDEALIZATION,
        nTemp: Optional[int] = None,
        thicknessModulus: Optional[float] = None,
        useDensity: Boolean = OFF,
        density: float = 0,
        layupName: str = "",
        thicknessField: str = "",
        nodalThicknessField: str = "",
    ) -> CompositeShellSection:
        """This method creates a CompositeShellSection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].CompositeLayup

        Parameters
        ----------
        name
            A String specifying the repository key.
        layup
            A SectionLayerArray object specifying the shell cross-section.
        symmetric
            A Boolean specifying whether or not the layup should be made symmetric by the analysis.
            The default value is OFF.
        thicknessType
            A SymbolicConstant specifying the distribution used for defining the thickness of the
            elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
            NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
        preIntegrate
            A Boolean specifying whether the shell section properties are specified by the user
            prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
            OFF.
        poissonDefinition
            A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
            Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
            is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
            Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
            is the value provided in **poisson**.The default value is DEFAULT.
        poisson
            A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
            This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
        integrationRule
            A SymbolicConstant specifying the shell section integration rule. Possible values are
            SIMPSON and GAUSS. The default value is SIMPSON.
        temperature
            A SymbolicConstant specifying the mode used for temperature and field variable input
            across the section thickness. Possible values are GRADIENT and POINTWISE. The default
            value is GRADIENT.
        idealization
            A SymbolicConstant specifying the mechanical idealization used for the section
            calculations. This member is only applicable when **preIntegrate** is set to ON. Possible
            values are NO_IDEALIZATION, SMEAR_ALL_LAYERS, MEMBRANE, and BENDING. The default value
            is NO_IDEALIZATION.
        nTemp
            None or an Int specifying the number of temperature points to be input. This argument is
            valid only when **temperature** = POINTWISE. The default value is None.
        thicknessModulus
            None or a Float specifying the effective thickness modulus. This argument is relevant
            only for continuum shells and must be used in conjunction with the argument **poisson**.
            The default value is None.
        useDensity
            A Boolean specifying whether or not to use the value of **density**. The default value is
            OFF.
        density
            A Float specifying the value of density to apply to this section. The default value is
            0.0.
        layupName
            A String specifying the layup name for this section. The default value is an empty
            string.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements. The **thicknessField** argument applies only
            when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
            value is an empty string.
        nodalThicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements at each node. The **nodalThicknessField**
            argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
            **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.

        Returns
        -------
        CompositeShellSection
            A CompositeShellSection object.
        """
        self.section = compositeShellSection = CompositeShellSection(
            name,
            layup,
            symmetric,
            thicknessType,
            preIntegrate,
            poissonDefinition,
            poisson,
            integrationRule,
            temperature,
            idealization,
            nTemp,
            thicknessModulus,
            useDensity,
            density,
            layupName,
            thicknessField,
            nodalThicknessField,
        )
        return compositeShellSection

    @abaqus_method_doc
    def GeometryShellSection(
        self,
        nodalThicknessField: str = "",
        thicknessField: str = "",
        thicknessType: Literal[
            C.DISCRETE_FIELD, C.NODAL_ANALYTICAL_FIELD, C.ANALYTICAL_FIELD, C.UNIFORM, C.NODAL_DISCRETE_FIELD
        ] = UNIFORM,
        preIntegrate: Boolean = OFF,
        poissonDefinition: Literal[C.DEFAULT] = DEFAULT,
        poisson: float = 0,
        integrationRule: Literal[C.GAUSS, C.SIMPSON] = SIMPSON,
        temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT,
        nTemp: Optional[int] = None,
        thicknessModulus: Optional[float] = None,
        useDensity: Boolean = OFF,
        density: float = 0,
    ) -> GeometryShellSection:
        """This method creates a GeometryShellSection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].CompositeLayup

        Parameters
        ----------
        nodalThicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements at each node. The **nodalThicknessField**
            argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
            **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements. The **thicknessField** argument applies only
            when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
            value is an empty string.
        thicknessType
            A SymbolicConstant specifying the distribution used for defining the thickness of the
            elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
            NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
        preIntegrate
            A Boolean specifying whether the shell section properties are specified by the user
            prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
            OFF.
        poissonDefinition
            A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
            Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
            is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
            Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
            is the value provided in **poisson**.The default value is DEFAULT.
        poisson
            A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
            This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
        integrationRule
            A SymbolicConstant specifying the shell section integration rule. Possible values are
            SIMPSON and GAUSS. The default value is SIMPSON.
        temperature
            A SymbolicConstant specifying the mode used for temperature and field variable input
            across the section thickness. Possible values are GRADIENT and POINTWISE. The default
            value is GRADIENT.
        nTemp
            None or an Int specifying the number of temperature points to be input. This argument is
            valid only when **temperature** = POINTWISE. The default value is None.
        thicknessModulus
            None or a Float specifying the effective thickness modulus. This argument is relevant
            only for continuum shells and must be used in conjunction with the argument **poisson**.
            The default value is None.
        useDensity
            A Boolean specifying whether or not to use the value of **density**. The default value is
            OFF.
        density
            A Float specifying the value of density to apply to this section. The default value is
            0.0.

        Returns
        -------
        GeometryShellSection
            A GeometryShellSection object.
        """
        self.section = geometryShellSection = GeometryShellSection(
            nodalThicknessField,
            thicknessField,
            thicknessType,
            preIntegrate,
            poissonDefinition,
            poisson,
            integrationRule,
            temperature,
            nTemp,
            thicknessModulus,
            useDensity,
            density,
        )
        return geometryShellSection

    @abaqus_method_doc
    def HomogeneousShellSection(
        self,
        name: str,
        material: str,
        thickness: float = 0,
        numIntPts: int = 5,
        thicknessType: Literal[
            C.DISCRETE_FIELD, C.NODAL_ANALYTICAL_FIELD, C.ANALYTICAL_FIELD, C.UNIFORM, C.NODAL_DISCRETE_FIELD
        ] = UNIFORM,
        preIntegrate: Boolean = OFF,
        poissonDefinition: Literal[C.DEFAULT] = DEFAULT,
        poisson: float = 0,
        integrationRule: Literal[C.GAUSS, C.SIMPSON] = SIMPSON,
        temperature: Literal[C.GRADIENT, C.POINTWISE] = GRADIENT,
        idealization: Literal[C.SMEAR_ALL_LAYERS, C.BENDING, C.MEMBRANE, C.NO_IDEALIZATION] = NO_IDEALIZATION,
        nTemp: Optional[int] = None,
        thicknessModulus: Optional[float] = None,
        useDensity: Boolean = OFF,
        density: float = 0,
        thicknessField: str = "",
        nodalThicknessField: str = "",
    ) -> HomogeneousShellSection:
        """This method creates a HomogeneousShellSection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].CompositeLayup

        Parameters
        ----------
        name
            A String specifying the repository key.
        material
            A String specifying the name of the section material.
        thickness
            A Float specifying the thickness of the section. The **thickness** argument applies only
            when **thicknessType** = UNIFORM. The default value is 0.0.
        numIntPts
            An Int specifying the number of integration points to be used through the section.
            Possible values are **numIntPts** > 0. The default value is 5.To use the default settings
            of the analysis products, set **numIntPts** to 5 if **integrationRule** = SIMPSON or set
            **numIntPts** to 7 if **integrationRule** = GAUSS.
        thicknessType
            A SymbolicConstant specifying the distribution used for defining the thickness of the
            elements. Possible values are UNIFORM, ANALYTICAL_FIELD, DISCRETE_FIELD,
            NODAL_ANALYTICAL_FIELD, and NODAL_DISCRETE_FIELD. The default value is UNIFORM.
        preIntegrate
            A Boolean specifying whether the shell section properties are specified by the user
            prior to the analysis (ON) or integrated during the analysis (OFF). The default value is
            OFF.
        poissonDefinition
            A SymbolicConstant specifying whether to use the default value for the Poisson's ratio.
            Possible values are:DEFAULT, specifying that the default value for the Poisson's ratio
            is 0.5 in an Abaqus/Standard analysis and is obtained from the material definition in an
            Abaqus/Explicit analysis.VALUE, specifying that the Poisson's ratio used in the analysis
            is the value provided in **poisson**.The default value is DEFAULT.
        poisson
            A Float specifying the Poisson's ratio. Possible values are −1.0 ≤ **poisson** ≤ 0.5.
            This argument is valid only when **poissonDefinition** = VALUE. The default value is 0.5.
        integrationRule
            A SymbolicConstant specifying the shell section integration rule. Possible values are
            SIMPSON and GAUSS. The default value is SIMPSON.
        temperature
            A SymbolicConstant specifying the mode used for temperature and field variable input
            across the section thickness. Possible values are GRADIENT and POINTWISE. The default
            value is GRADIENT.
        idealization
            A SymbolicConstant specifying the mechanical idealization used for the section
            calculations. This member is only applicable when **preIntegrate** is set to ON. Possible
            values are NO_IDEALIZATION, SMEAR_ALL_LAYERS, MEMBRANE, and BENDING. The default value
            is NO_IDEALIZATION.
        nTemp
            None or an Int specifying the number of temperature points to be input. This argument is
            valid only when **temperature** = POINTWISE. The default value is None.
        thicknessModulus
            None or a Float specifying the effective thickness modulus. This argument is relevant
            only for continuum shells and must be used in conjunction with the argument **poisson**.
            The default value is None.
        useDensity
            A Boolean specifying whether or not to use the value of **density**. The default value is
            OFF.
        density
            A Float specifying the value of density to apply to this section. The default value is
            0.0.
        thicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements. The **thicknessField** argument applies only
            when **thicknessType** = ANALYTICAL_FIELD or **thicknessType** = DISCRETE_FIELD. The default
            value is an empty string.
        nodalThicknessField
            A String specifying the name of the AnalyticalField or DiscreteField object used to
            define the thickness of the shell elements at each node. The **nodalThicknessField**
            argument applies only when **thicknessType** = NODAL_ANALYTICAL_FIELD or
            **thicknessType** = NODAL_DISCRETE_FIELD. The default value is an empty string.

        Returns
        -------
        HomogeneousShellSection
            A HomogeneousShellSection object.
        """
        self.section[name] = homogeneousShellSection = HomogeneousShellSection(
            name,
            material,
            thickness,
            numIntPts,
            thicknessType,
            preIntegrate,
            poissonDefinition,
            poisson,
            integrationRule,
            temperature,
            idealization,
            nTemp,
            thicknessModulus,
            useDensity,
            density,
            thicknessField,
            nodalThicknessField,
        )
        return homogeneousShellSection
